/*LICENSE_BEGIN
 * ******************************************************************
 * Copyright 2006 by the Dainamite Robocup-Team at the
 * DAI-Labor, TU-Berlin (www.dai-labor.de)
 * 
 * Members of Dainamite are:
 * Holger Endert
 * Robert Wetzker
 * Thomas Karbe
 * Axel Heßler
 * Philippe Bütter
 * Felix Brossmann
 * 
 * Licensed under the GNU GPL (General Public License) Free Software 
 * License, Version 2. You may use this file only in compliance with 
 * the License. You may obtain a copy of the License at:
 * 
 * http://www.fsf.org/licensing/licenses/gpl.txt
 * ******************************************************************
LICENSE_END*/
package robocup.component.infotypes;

import robocup.component.ControllerNumbers.TEAM;

/**
 * This class is an extension of the CoachSayInfo, which is received for clang
 * messages. Here, the team of the sender is added for identifying the messages
 * that are important.
 */
public class OLCoachSayInfo extends CoachSayInfo {

	/**
	 * The team of the sender. Either TEAM.WE or TEAM.OPPONENT.
	 */
	private TEAM side;

	/**
	 * Constructs an extension to the CoachSayInfo, which holds for instance
	 * CLANG Messages. Here, a team is added
	 * 
	 * @param iCycle -
	 *            the time of the message
	 */
	public OLCoachSayInfo(int iCycle) {
		super(iCycle);
	}

	/**
	 * set the side from which a coach said something
	 * 
	 * @param side
	 *            the side of the coach who said this.
	 */
	public void setSide(TEAM side) {
		this.side = side;
	}

	/**
	 * @return the side of the coach who said this.
	 */
	public TEAM getSide() {
		return this.side;
	}

}
