/*LICENSE_BEGIN
 * ******************************************************************
 * All original parts of SoccerScope2003 have copyright (2003) by
 * Team YowAI, represented by Mr. Shuhei Shiota
 * The Department of Creative Informatics
 * The Graduate School of Information Science and Technology
 * The University of Tokyo
 * 
 * Additional classes (except libraries in /lib) were developed by 
 * the Dainamite Robocup-Team of the DAI-Labor, TU-Berlin 
 * (www.dai-labor.de)
 * 
 * Extended/new classes/packages are:
 *   soccerscope.analyzer.LogfileAnalyzer
 *   soccerscope.daistatistic.*
 *   soccerscope.daistatistic.charts.*
 *   soccerscope.daistatistic.tools.*
 *   soccerscope.db.*
 *   soccerscope.dialog.*
 *   soccerscope.model.PlayerWorldModel
 *   soccerscope.model.SimplePlayer
 *   soccerscope.net.AgentConnector
 *   soccerscope.training.*
 *   soccerscope.training.conditions.*
 * ******************************************************************
LICENSE_END*/
package soccerscope;

import java.awt.Container;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.HeadlessException;
import java.awt.Toolkit;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.IOException;
import java.net.SocketException;
import java.net.UnknownHostException;
import java.util.prefs.Preferences;

import javax.swing.JFrame;

import soccerscope.daistatistic.Register;
import soccerscope.file.LogFileReader;
import soccerscope.model.ClientScene;
import soccerscope.model.SceneSet;
import soccerscope.model.SceneSetMaker;
import soccerscope.model.ServerSetMaker;
import soccerscope.model.SoccerScopePreferences;
import soccerscope.model.WorldModel;
import soccerscope.net.SoccerServerConnection;
import soccerscope.util.ButtonManager;
import soccerscope.view.InformationWindow;
import soccerscope.view.OptionWindow;
import soccerscope.view.ScenePlayer;
import soccerscope.view.ScopePane;
import soccerscope.view.SelectWindow;
import soccerscope.view.StatisticsWindow;
import soccerscope.view.StatusBar;
import soccerscope.db.RobocupDatabase;

public class SoccerScope extends JFrame {

	/**
	 *
	 */
	private static final long serialVersionUID = 3257562906245018931L;

	public final static String title = "SoccerScope";

	Container container = null;

	private File chooseFile = null;

	private File serverFile = null;

	private int maxX = Toolkit.getDefaultToolkit().getScreenSize().width;

	private int maxY = Toolkit.getDefaultToolkit().getScreenSize().height;

	// server connection
	private SoccerServerConnection ssConnection = null;

	// main window component
	private SoccerScopeMenuBar menuBar = null;

	private SoccerScopeToolBar toolBar = null;

	private ScopePane scopePane = null;

	private StatusBar statusBar = null;

    private ScenePlayer scenePlayer = null;

	// select window
	private SelectWindow selectWindow = null;

	// information window
	private InformationWindow infoWindow = null;

	// Statistics window
	private StatisticsWindow statisticsWindow = null;

	// option window
	private OptionWindow optionWindow = null;

	private static SoccerScope soccerScope;


	public static synchronized SoccerScope getInstance() {
		if (soccerScope == null) {
			soccerScope = new SoccerScope();
		}
		return soccerScope;
	}

	/**
	 * Constructs a new frame that is initially invisible. <p/> This constructor
	 * sets the component's locale property to the value returned by
	 * <code>JComponent.getDefaultLocale</code>.
	 *
	 * @throws java.awt.HeadlessException
	 *             if GraphicsEnvironment.isHeadless() returns true.
	 * @see java.awt.GraphicsEnvironment#isHeadless
	 * @see java.awt.Component#setSize
	 * @see java.awt.Component#setVisible
	 * @see javax.swing.JComponent#getDefaultLocale
	 */
	public SoccerScope() throws HeadlessException {
		this(null);

	}

	public SoccerScope(String[] args) {
		// frame title
		super("SoccerScope DAI Labor Edition");
		soccerScope = this;

		Register.soccerScope = this;

		// get content pane
		this.container = this.getContentPane();
		this.container.setLayout(new GridBagLayout());

		// init mainpanel (ScopePane, fieldPane, infoBar)
		initMainPanel();

		// init MenuBar (menuBar)
		initMenuBar();

		// init statusBar (statusBar)
		initStatusBar(true);

		// init ToolBar (toolBar, scenePlayer)
		initToolBar(true);
		initScenePlayer(true);

		// init all Button's state
		initButtonState();

		// ScenePlayer ScopePane˴ϢŤ
		this.scopePane.setScenePlayer(this.scenePlayer);

		// WindowListener
		addWindowListener(new WindowEventHander());

		// realize
		pack();
		setSize(1000, 750);
		setVisible(true);

		// appear center of screen
		setLocationRelativeTo(null);
		setLocation(20, 20);

		try {
			// try to connect to localhost
			setSoccerServerConnection(new SoccerServerConnection());
			SceneSetMaker ssm = new SceneSetMaker(getSoccerServerConnection(),
					WorldModel.getSceneSet());
			try {
				getSoccerServerConnection().dispinit();
				Thread.sleep(1000);
				getScenePlayer().sync();
				ssm.start();
				ButtonManager.setEnabled("network", true);
			} catch (InterruptedException iie) {
			} catch (IOException ie) {
				System.err.println(ie);
			}
		} catch (SocketException se) {
			System.err.println(se);
		} catch (UnknownHostException uhe) {
			System.err.println(uhe);
		}
	}

	private void initMainPanel() {
		this.scopePane = new ScopePane(this);
		// Add listeners for select player event

		GridBagConstraints gbc = new GridBagConstraints();
		gbc.gridx = 0;
		gbc.gridy = 3;
		gbc.gridwidth = 1;
		gbc.gridheight = 1;
		gbc.weightx = 1.0;
		gbc.weighty = 1.0;
		gbc.fill = GridBagConstraints.BOTH;
		gbc.anchor = GridBagConstraints.NORTH;
		this.container.add(this.scopePane, gbc);
	}

	private void initMenuBar() {
		this.menuBar = new SoccerScopeMenuBar(this);
		setJMenuBar(this.menuBar);
	}

	public void initToolBar(boolean enable) {
		if (enable) {
			GridBagConstraints gbc = new GridBagConstraints();
			gbc.gridx = 0;
			gbc.gridy = 0;
			gbc.gridwidth = 1;
			gbc.gridheight = 1;
			gbc.weightx = 1.0;
			gbc.weighty = 0.0;
			gbc.fill = GridBagConstraints.HORIZONTAL;
			gbc.anchor = GridBagConstraints.SOUTHWEST;
			if (this.toolBar == null) {
				this.toolBar = new SoccerScopeToolBar(this);
				this.container.add(this.toolBar, gbc);
			} else {
				this.container.add(this.toolBar, gbc);
				pack();
				setVisible(true);
			}
		} else {
			this.container.remove(this.toolBar);
			pack();
			setVisible(true);
		}
	}

	public void initScenePlayer(boolean enable) {
		if (enable) {
			GridBagConstraints gbc = new GridBagConstraints();
			gbc.gridx = 0;
			gbc.gridy = 1;
			gbc.gridwidth = 1;
			gbc.gridheight = 1;
			gbc.weightx = 1.0;
			gbc.weighty = 0.0;
			gbc.fill = GridBagConstraints.HORIZONTAL;
			gbc.anchor = GridBagConstraints.SOUTHWEST;
			if (this.scenePlayer == null) {
				this.scenePlayer = new ScenePlayer(this.scopePane, this.statusBar, true);
				this.scenePlayer.setFloatable(false);
				this.container.add(this.scenePlayer, gbc);
			} else {
				this.container.add(this.scenePlayer, gbc);
				pack();
				setVisible(true);
			}
		} else {
			this.container.remove(this.scenePlayer);
			pack();
			setVisible(true);
		}
	}

	public void initStatusBar(boolean enable) {
		if (enable) {
			GridBagConstraints gbc = new GridBagConstraints();
			gbc.gridx = 0;
			gbc.gridy = 4;
			gbc.gridwidth = 1;
			gbc.gridheight = 1;
			gbc.weightx = 1.0;
			gbc.weighty = 0.0;
			gbc.fill = GridBagConstraints.HORIZONTAL;
			gbc.anchor = GridBagConstraints.SOUTHWEST;
			if (this.statusBar == null) {
				this.statusBar = new StatusBar(this.scopePane);
				this.container.add(this.statusBar, gbc);
			} else {
				this.container.add(this.statusBar, gbc);
				pack();
				setVisible(true);
			}
		} else {
			this.container.remove(this.statusBar);
			pack();
			setVisible(true);
		}
	}

	private void initButtonState() {
		ButtonManager.setEnabled("logfile", true);
		ButtonManager.setEnabled("AgentWorldModel", true);
		ButtonManager.setEnabled("AgentLog", true);
		ButtonManager.setEnabled("network", true);
	}

	public void openLogFile(SceneSet sceneSet, String filename) {
		try {
			LogFileReader lfr = new LogFileReader(filename);
			SceneSetMaker ssm = new SceneSetMaker(lfr, sceneSet, this.statusBar
					.getProgressBar());
			ssm.start();
		} catch (IOException ie) {
			System.err.println(ie);
		}
	}

	public void openServerLogFile(SceneSet sceneSet, String filename) {
		try {
			LogFileReader lfr = new LogFileReader(filename);
			ServerSetMaker ssm = new ServerSetMaker(lfr, sceneSet);
			ssm.start();
		} catch (IOException ie) {
			System.err.println(ie);
		}
	}

	class WindowEventHander extends WindowAdapter {
		@Override
    public void windowClosing(WindowEvent evt) {
			JFrame f = (JFrame) evt.getSource();
			if (f.getTitle().startsWith(SoccerScope.title)) {
				// System.exit(0);
				f.setVisible(false);
				f.dispose();
			} else
				ButtonManager.setSelected(f.getTitle(), false);
		}
	}

	public SoccerScopeMenuBar getSoccerScopeMenuBar() {
		return this.menuBar;
	}

	public SoccerScopeToolBar getSoccerScopeToolBar() {
		return this.toolBar;
	}

	public ScopePane getScopePane() {
		return this.scopePane;
	}

	public ScenePlayer getScenePlayer() {
		return this.scenePlayer;
	}

	public SelectWindow getSelectWindow() {
		if (this.selectWindow == null) {
			this.selectWindow = new SelectWindow();
			this.selectWindow.addWindowListener(new WindowEventHander());
			this.scenePlayer.addScopeWindow(this.selectWindow);
			// ScopePane
			this.scopePane.addSoccerObjectSelectObserver(this.selectWindow);

			// SelectWindow
			this.selectWindow.addSoccerObjectSelectObserver(this.scopePane);
			if (this.infoWindow != null)
				this.selectWindow.addSoccerObjectSelectObserver(this.infoWindow);
			// и֤
			this.selectWindow.setLocation(Math.min(this.maxX - this.selectWindow.getWidth(),
					getLocationOnScreen().x + getWidth()), Math.min(this.maxY
					- this.selectWindow.getHeight(), getLocationOnScreen().y
					+ getHeight() - this.selectWindow.getHeight()));
		}
		return this.selectWindow;
	}

	public void setSelectWindow(SelectWindow sw) {
		this.selectWindow = sw;
	}

	public InformationWindow getInformationWindow() {
		if (this.infoWindow == null) {
			this.infoWindow = new InformationWindow();
			this.infoWindow.addWindowListener(new WindowEventHander());
			this.scenePlayer.addScopeWindow(this.infoWindow);
			// ScopePane
			this.scopePane.addSoccerObjectSelectObserver(this.infoWindow);

			// SelectWindow
			if (this.selectWindow != null)
				this.selectWindow.addSoccerObjectSelectObserver(this.infoWindow);
			// и֤
			this.infoWindow.setLocation(Math.min(this.maxX - this.infoWindow.getWidth(),
					getLocationOnScreen().x + getWidth()
							- this.infoWindow.getWidth()), Math.min(this.maxY
					- this.infoWindow.getHeight(), getLocationOnScreen().y
					+ getHeight()));
		}
		return this.infoWindow;
	}

	public void setInformationWindow(InformationWindow sw) {
		this.infoWindow = sw;
	}

	public StatisticsWindow getStatisticsWindow() {
		if (this.statisticsWindow == null) {
			this.statisticsWindow = new StatisticsWindow();
			this.statisticsWindow.addWindowListener(new WindowEventHander());

			this.statisticsWindow.setLocation(Math.min(this.maxX
					- this.statisticsWindow.getWidth(), getLocationOnScreen().x),
					Math.min(this.maxY - this.statisticsWindow.getHeight(),
							getLocationOnScreen().y + getHeight()));
		}
		return this.statisticsWindow;
	}

	public void setStatisticsWindow(StatisticsWindow sw) {
		this.statisticsWindow = sw;
	}

	public OptionWindow getOptionWindow() {
		if (this.optionWindow == null) {
			this.optionWindow = new OptionWindow(this.scopePane);
			this.optionWindow.addWindowListener(new WindowEventHander());
			this.scenePlayer.addScopeWindow(this.optionWindow);
			this.optionWindow.setLocation(Math.min(this.maxX - this.optionWindow.getWidth(),
					getLocationOnScreen().x + getWidth()), Math.min(this.maxY
					- this.optionWindow.getHeight(), getLocationOnScreen().y));
		}
		return this.optionWindow;
	}

	public void setOptionWindow(OptionWindow sw) {
		this.optionWindow = sw;
	}

	public SoccerServerConnection getSoccerServerConnection() {
		return this.ssConnection;
	}

	public void setSoccerServerConnection(SoccerServerConnection ssc) {
		this.ssConnection = ssc;
	}

	public File getChooseFile() {
		if (this.chooseFile == null) {
			Preferences pf = SoccerScopePreferences.getPreferences();
			this.chooseFile = new File(pf.get("lastfile", System
					.getProperty("user.dir")));
		}
		return this.chooseFile;
	}

	public void setChooseFile(File f) {
		this.chooseFile = f;
		Preferences pf = SoccerScopePreferences.getPreferences();
		pf.put("lastfile", f.getPath());
	}

	public File getChooseServerFile() {
		if (this.serverFile == null) {
			Preferences pf = SoccerScopePreferences.getPreferences();
			this.serverFile = new File(pf.get("lastserverfile", System
					.getProperty("user.dir")));
		}
		return this.serverFile;
	}

	public void setChooseServerFile(File f) {
		this.serverFile = f;
		Preferences pf = SoccerScopePreferences.getPreferences();
		pf.put("lastserverfile", f.getPath());
	}

	public String getServerFile() {
		if (this.chooseFile == null)
			return null;
		String filename = this.chooseFile.getName();
		filename = filename.substring(0, filename.indexOf("."));
		return filename;
	}

	public String getDataSourceName() {
		if (this.ssConnection != null)
			return this.ssConnection.getHostName() + ":" + this.ssConnection.getPort();
		if (this.chooseFile != null)
			return this.chooseFile.getName();
		return "scene";
	}

	/**
	 * This method adds a say-message to the actual scene, choosing the player
	 *
	 * @param msg -
	 *            the message
	 */
	public void addSayMessageToScene(int pIndex, String msg) {
		if (pIndex >= 0 && pIndex < 22) {
			if (WorldModel.getSceneSet().lastScene() != null) {
				WorldModel.getSceneSet().lastScene().player[pIndex].sayMessage = msg;
			}
		}
	}

	/**
	 * This method adds a perceived-message (hear) to the actual scene, choosing
	 * the player by his id
	 *
	 * @param msg -
	 *            the message
	 */
	public void addHearMessageToScene(int pIndex, String msg) {
		if (pIndex >= 0 && pIndex < 22) {
			ClientScene scene = WorldModel.getSceneSet()
					.lastScene();
			if (scene != null)
				scene.player[pIndex].hearMessage.add(msg);
		}
	}

        public StatusBar getStatusBar() {
            return statusBar;
        }
    }
/**
 * $Log: SoccerScope.java,v $
 * Revision 1.1.28.5  2006/11/23 09:41:14  endert
 * Added License Note at top of each *.java file.
 *
 * Revision 1.1.28.4  2006/11/13 16:50:22  endert
 * Removed CVS-Logs and author tags
 *
 * Revision 1.1.28.3  2006/10/25 14:33:35  endert
 * Latest SoccerScope Version plus Database ability
 *
 * Revision 1.1.14.5  2006/10/09 00:46:07  lphle
 * nicht benuzte Variable rcDatabase gelscht
 *
 * Revision 1.1.14.4  2006/08/14 13:43:48  endert
 * Replaced Jump-Editors to ScenePlayer and Removed Jump..Toolbar
 *
 * Revision 1.1.14.3  2006/07/25 12:46:07  benjaboe
 * Jump list filtered by play modes available
 *
 * Revision 1.1.14.2  2006/07/18 15:48:50  lphle
 * neue Methode zum Zugreifen auf progressbar
 *
 * Revision 1.1.14.1  2006/07/12 16:02:03  lphle
 * Datenbank eingebaut
 *
 * Revision 1.1  2006/03/13 07:38:26  axle
 * moved from ROOT/src for Maven 2 Buildsystem and Team/Soccerscope separation
 *
 * Revision 1.10  2006/03/07 12:46:40  endert
 * Merge after refactoring:
 *  - New State-Interface: One Method for each action type
 *  - Action Bean removed - ActionFactorys for helper methods, States calculate
 *    main-Action themselves
 *  - formatted code
 *  - added State-List in Monitor (Context-Menu of Player)
 *  - other smaller stuff
 *
 * Revision 1.9.2.2  2006/03/06 23:17:54  endert
 * Removed warnings, added State-Overview to Context-Menu!
 *
 * Revision 1.9.2.1  2006/02/25 22:07:29  wetzker
 * formated
 *
 * Revision 1.9  2006/02/22 16:05:12  endert
 * This is done:
 * - Rebound SoccerScope to Agents (can be added or removed in config-file)
 * - Situations now all extend abstract class Situation
 * - Refactorings: ActionBean _ removed from Attributes
 * - SoccerScope will automatically connect to host given in Start-Call
 * - Trainer uses the same host as soccerscope
 * - Removed unused stuff (old confs, bat, etc.)
 * - Code formatted (good luck when merging)
 * - organized imports
 * - all that i forgot to mention here
 * Revision 1.8 2006/02/06 17:09:45 endert New Communication merged, bla
 * bla
 *
 * Revision 1.7.32.2 2006/01/02 13:40:20 endert Changes and implementation for
 * coordinated defense
 *
 * Revision 1.7.32.1 2005/12/27 19:35:58 endert New Branch created for my Dipl!
 *
 * Revision 1.7 2005/10/25 20:51:43 endert Added Training-Editor to SoccerScope,
 * restructuring in some old classes and some experimental stuff (goals,
 * commitments, mental model)
 *
 * Revision 1.6 2005/10/14 14:58:15 endert Now, say-messages are visualized
 * inside in SoccerScope. However, the information comes from the RobocupAgent
 * directly, using the SoccerScope.getInstance() method (in GUI-Mode only), so
 * that it is a work-around. The message itself can be seen inside the
 * "Property"-Menu of the Players Context-Menu, as well as any heared PSI,
 * whereas saying is visualized additionally on the layer (String removed, was
 * to big! Uncomment the drawStrings in SayLayer, to revisualize them).
 *
 * Revision 1.5 2005/06/15 17:03:38 wetzker removed JIAC from Head + changes in
 * framework please report bugs or shortcomings some scenarios might not work
 * because of wrong imports
 *
 * Revision 1.2.2.2 2005/06/15 16:44:56 wetzker final version
 *
 * Revision 1.2.2.1 2005/06/10 14:34:41 endert debugging
 *
 * Revision 1.4 2005/06/13 12:08:06 andpaul added: register soccerscope to
 * daistatistics.Register for direct acces to soccerscope fields/objects
 *
 * Revision 1.3 2005/06/12 18:52:18 thyrion fixed saving&loading of our
 * worldmodel (to load first open server log)
 *
 * Revision 1.2 2005/05/05 22:51:11 smarqoo GUI attempts to connect to localhost
 * at start
 *
 * Revision 1.1 2005/04/20 10:39:43 endert Fresh Team Repository installation
 * from /home/e/endert/Robocup to here.
 *
 * Revision 1.8 2005/04/02 11:08:19 andpaul documentation
 *
 * Revision 1.7 2005/03/20 21:41:09 andpaul removed some warnings
 *
 * Revision 1.6 2005/03/12 14:19:39 andpaul added exit SoccerScope and exit
 * platform added dropBall (drops the ball to its current position) and dropBall
 * to Center (shortcut STRG-C, drops the ball to the center position)
 *
 *
 *
 *
 */
