/*LICENSE_BEGIN
 * ******************************************************************
 * All original parts of SoccerScope2003 have copyright (2003) by
 * Team YowAI, represented by Mr. Shuhei Shiota
 * The Department of Creative Informatics
 * The Graduate School of Information Science and Technology
 * The University of Tokyo
 * 
 * Additional classes (except libraries in /lib) were developed by 
 * the Dainamite Robocup-Team of the DAI-Labor, TU-Berlin 
 * (www.dai-labor.de)
 * 
 * Extended/new classes/packages are:
 *   soccerscope.analyzer.LogfileAnalyzer
 *   soccerscope.daistatistic.*
 *   soccerscope.daistatistic.charts.*
 *   soccerscope.daistatistic.tools.*
 *   soccerscope.db.*
 *   soccerscope.dialog.*
 *   soccerscope.model.PlayerWorldModel
 *   soccerscope.model.SimplePlayer
 *   soccerscope.net.AgentConnector
 *   soccerscope.training.*
 *   soccerscope.training.conditions.*
 * ******************************************************************
LICENSE_END*/
package soccerscope.db;

import soccerscope.model.ClientScene;
import java.sql.Connection;
import soccerscope.model.SceneSet;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import soccerscope.model.Param;
import soccerscope.model.PlayerWorldModel;
import java.util.ArrayList;
import java.sql.Statement;
import javax.swing.JProgressBar;

/**
 * <p>Title: </p>
 *
 * <p>Description: </p>
 *
 * <p>Copyright: Copyright (c) 2006</p>
 *
 * <p>Company: </p>
 *
 * @author Linh Phong Le
 * @version 1.0
 */
public class DatabaseReader extends Thread {
    public DatabaseReader() {
    }

    private static DatabaseReader dbReader = null;

    // Zugang zu Datenbank-Server
    public static Connection connection = null;
    public String sqlHost;
    public String sqlDB;
    public String sqlUser;
    public String sqlPassword;

    public static SceneSet sceneSet = null;
    public JProgressBar progress = null;

    // Anfragen
    private String squery = "SELECT * FROM client_scene WHERE game_id = ?";
    private PreparedStatement sstm = null;

    private String tquery = "SELECT * FROM team WHERE scene_id = ?";
    private PreparedStatement tstm = null;

    private String pquery =
    "SELECT * FROM player WHERE scene_id = ?";
    private PreparedStatement pstm = null;

    private String pwmquery =
    "SELECT * FROM player_world_model WHERE player_id = ?";
    private PreparedStatement pwmstm = null;

    private String spquery = "SELECT * FROM simple_player WHERE pwm_id = ?";
    private PreparedStatement spstm = null;

    final private int CPU_SAVING_DELAY = 2;


    public static int gameID = -1;
/*    private static int clientSceneID = -1;
    private static int playerID = -1;
    private static int pwmID = -1;
    private static int simplePlayerID = -1;
*/
    public DatabaseReader(SceneSet sceneSet) {
        try {
            DatabaseConfig dbConfig = DatabaseConfig.getInstance();
            dbConfig.load();
            sqlHost = dbConfig.getSqlHost();
            sqlDB = dbConfig.getSqlDB();
            sqlUser = dbConfig.getSqlUser();
            sqlPassword = dbConfig.getSqlPassword();

            Class.forName("com.mysql.jdbc.Driver").newInstance();
            connection = DriverManager.getConnection("jdbc:mysql://" + sqlHost + "/" + sqlDB, sqlUser, sqlPassword);

            DatabaseReader.sceneSet = sceneSet;
            System.out.println("[RobocupDatabase] Database connected");

            // vorbereitet die Anfragen fr Batch-Arbeit
            sstm = connection.prepareStatement(squery);
            sstm.setFetchDirection(ResultSet.FETCH_FORWARD);
            sstm.setFetchSize(100);

            tstm = connection.prepareStatement(tquery);
            tstm.setFetchDirection(ResultSet.FETCH_FORWARD);
            tstm.setFetchSize(200);

            pstm = connection.prepareStatement(pquery);
            pstm.setFetchDirection(ResultSet.FETCH_FORWARD);
            pstm.setFetchSize(500);

            pwmstm = connection.prepareStatement(pwmquery);
            pwmstm.setFetchSize(250);
            spstm = connection.prepareStatement(spquery);
            spstm.setFetchSize(25);
/*
            // Holt IDs fr nchste Records
            clientSceneID = getNextID("client_scene");
            playerID = getNextID("player");
            pwmID = getNextID("player_world_model");

            saveGame();
*/
        } catch (ClassNotFoundException ex) {
            ex.printStackTrace();
        } catch (IllegalAccessException ex) {
            ex.printStackTrace();
        } catch (InstantiationException ex) {
            ex.printStackTrace();
        } catch (SQLException ex) {
            ex.printStackTrace();
        }

    }

    public static synchronized DatabaseReader getInstance(SceneSet sceneSet) {
        if (null == dbReader)
            dbReader = new DatabaseReader(sceneSet);

        if (connection == null)
            dbReader = null;

        return dbReader;
    }

    public static synchronized DatabaseReader getInstance() {
        return dbReader;
    }

    /**
     * Initialisiert die Instanz nochmal, nachdem Datenbankinfo geaendert wurde
     */
    public static synchronized void restartInstance()
    {
        try {
            connection.close();
        } catch (SQLException ex) {
            ex.printStackTrace();
        }

        dbReader = null;
        dbReader = new DatabaseReader(sceneSet);
    }

    public void makeSceneSet()
    {
        sceneSet.clear();

        if (this.progress != null) {
            try {
                Statement stm = connection.createStatement();
                ResultSet rs = stm.executeQuery("SELECT * FROM game WHERE id = " + gameID);

                if (rs.next())
                    this.progress.setMaximum(rs.getInt("scene_number") - 1);

                rs.close();
                stm.close();
            } catch (SQLException ex) {
                ex.printStackTrace();
            }
            this.progress.setMinimum(0);
            this.progress.setValue(0);
        }
        int sceneCounter = 0;


        try {
            sstm.setInt(1, gameID);
            ResultSet rs = sstm.executeQuery();

            while (rs.next())
            {
                long tstart = System.currentTimeMillis();

                ClientScene clientScene = new ClientScene();

                int sceneID = rs.getInt("id");

                clientScene.pmode.pmode = rs.getInt("play_mode");
                clientScene.time = rs.getInt("time");

                clientScene.ball.pos.setX(rs.getFloat("ball_pos_x"));
                clientScene.ball.pos.setY(rs.getFloat("ball_pos_y"));
                clientScene.ball.vel.setX(rs.getFloat("ball_vel_x"));
                clientScene.ball.vel.setY(rs.getFloat("ball_vel_y"));
                clientScene.ball.acc.setX(rs.getFloat("ball_accX"));
                clientScene.ball.acc.setY(rs.getFloat("ball_accY"));
                clientScene.ball.mode = rs.getInt("ball_mode");

//                clientScene.ball.bestPos.setX(rs.getFloat("ball_bestPosX"));
//                clientScene.ball.bestPos.setY(rs.getFloat("ball_bestPosY"));

                tstm.setInt(1, sceneID);
                ResultSet trs = tstm.executeQuery();
                if (trs.next())
                {
                    clientScene.left.name = trs.getString("name");
                    clientScene.left.offside = trs.getBoolean("offside");
                    clientScene.left.offsideline = trs.getFloat("offside_line");
                    clientScene.left.score = trs.getInt("score");
                }
                if (trs.next())
                {
                    clientScene.right.name = trs.getString("name");
                    clientScene.right.offside = trs.getBoolean("offside");
                    clientScene.right.offsideline = trs.getFloat("offside_line");
                    clientScene.right.score = trs.getInt("score");
                }
                trs.close();


                pstm.setInt(1, sceneID);
                ResultSet prs = pstm.executeQuery();
                // int fz = prs.getFetchSize();
                for (int i = 0; i < Param.MAX_PLAYER * 2; i++) {
                    if (!prs.next())
                        break;
                    int playerID = prs.getInt("id");

                    clientScene.player[i].unum = prs.getInt("unum");
                    clientScene.player[i].pos.setX(prs.getFloat("pos_x"));
                    clientScene.player[i].pos.setY(prs.getFloat("pos_y"));
                    clientScene.player[i].vel.setX(prs.getFloat("vel_x"));
                    clientScene.player[i].vel.setY(prs.getFloat("vel_y"));
                    clientScene.player[i].acc.setX(prs.getFloat("acc_x"));
                    clientScene.player[i].acc.setY(prs.getFloat("acc_y"));
                    clientScene.player[i].side = prs.getInt("side");
                    clientScene.player[i].stamina = prs.getInt("stamina");
                    clientScene.player[i].angle = prs.getFloat("angle");
                    clientScene.player[i].angleNeck = prs.getInt("angle_neck");
                    clientScene.player[i].angleVisible = prs.getInt("angle_visible");
                    clientScene.player[i].viewQuality = prs.getInt("view_quality");
                    clientScene.player[i].sayMessage = prs.getString("say_message");
                    clientScene.player[i].catchDir = prs.getFloat("catch_dir");
                    clientScene.player[i].offside = prs.getBoolean("offside");
                    clientScene.player[i].mode = prs.getInt("mode");
                    clientScene.player[i].type = prs.getInt("type");
                    clientScene.player[i].playerType = prs.getInt("player_type");
                    clientScene.player[i].interceptionTime = prs.getInt("interception_time");
                    clientScene.player[i].roleNumber = prs.getInt("role_number");
                    clientScene.player[i].kickCount = prs.getInt("kick_count");
                    clientScene.player[i].dashCount = prs.getInt("dash_count");
                    clientScene.player[i].turnCount = prs.getInt("turn_count");
                    clientScene.player[i].sayCount = prs.getInt("say_count");
                    clientScene.player[i].turnNeckCount = prs.getInt("turn_neck_count");
                    clientScene.player[i].catchCount = prs.getInt("catch_count");
                    clientScene.player[i].moveCount = prs.getInt("move_count");
                    clientScene.player[i].changeViewCount = prs.getInt("change_view_count");
                    /**
                      @todo hearMessage
                    clientScene.player[i].hearMessage.toString()
                     */
                    clientScene.player[i].state = prs.getString("state");
                    clientScene.player[i].reliable = prs.getBoolean("reliable");

                    if (i > Param.MAX_PLAYER) // diese Spieler sind nicht aus unserem Team
                        continue;

                    pwmstm.setInt(1, playerID);
                    ResultSet pwmrs = pwmstm.executeQuery();
                    if (!pwmrs.next())
                        continue;

                    PlayerWorldModel pwm = new PlayerWorldModel();
                    pwm.positionX = pwmrs.getFloat("positionX");
                    pwm.positionY = pwmrs.getFloat("positionY");
                    pwm.positionIsReliable = pwmrs.getBoolean("positionIsReliable");
                    pwm.isGoalie = pwmrs.getBoolean("isGoalie");
                    pwm.simpleBody = pwmrs.getFloat("simpleBody");
                    pwm.speedX = pwmrs.getFloat("speedX");
                    pwm.speedY = pwmrs.getFloat("speedY");
                    pwm.ballInterceptionTime = pwmrs.getInt("ballInterceptionTime");
                    pwm.stamina = pwmrs.getInt("stamina");
                    pwm.playerType = pwmrs.getInt("playerType");
//                    pwm.lastState = pwmrs.getInt("lastState");
                    pwm.roleNumber = pwmrs.getInt("roleNumber");
//                    pwm.bestPos.getX() = pwmrs.getFloat("");
//                    pwm.bestPos.getY() = pwmrs.getFloat("");

                    pwm.neckAngle = pwmrs.getFloat("neckAngle");
                    pwm.bodyAngle = pwmrs.getFloat("bodyAngle");
                    pwm.ballPositionX = pwmrs.getFloat("ballPositionX");
                    pwm.ballPositionY = pwmrs.getFloat("ballPositionY");
                    pwm.ballSpeedX = pwmrs.getFloat("ballSpeedX");
                    pwm.ballSpeedY = pwmrs.getFloat("ballSpeedY");
                    pwm.ballInterceptX = pwmrs.getFloat("ballInterceptX");
                    pwm.ballInterceptY = pwmrs.getFloat("ballInterceptY");
                    pwm.oppNum = pwmrs.getInt("oppNum");
                    pwm.markAreaX = pwmrs.getFloat("markAreaX");
                    pwm.markAreaY = pwmrs.getFloat("markAreaY");
                    pwm.markAreaRad = pwmrs.getFloat("markAreaRad");
                    pwm.shouldMark = pwmrs.getBoolean("shouldMark");
                    pwm.oppInterceptX = pwmrs.getFloat("oppInterceptX");
                    pwm.oppInterceptY = pwmrs.getFloat("oppInterceptY");
                    pwm.offsideY = pwmrs.getFloat("offsideY");
                    pwm.doneAPass = pwmrs.getBoolean("doneAPass");
                    pwm.passToPlayer = pwmrs.getFloat("passToPlayer");
                    pwm.holes = pwmrs.getInt("holes");
                    pwm.based_on_vi = pwmrs.getBoolean("based_on_vi");
//                    pwm.stateInfos.toString() = pwmrs.getFloat(""); //

                    clientScene.player[i].setWorldModel(pwm);

                    // int pwmid = pwmrs.getInt("id");
                    pwmrs.close();

//                    long sps = System.currentTimeMillis();
 //                   spstm.setInt(1, pwmid);
//                    ResultSet sprs = spstm.executeQuery();
//                    System.out.println("[SimplePlayer Query] " + (System.currentTimeMillis() - sps));

//                    while (sprs.next());
  //                  sprs.close();

                }

                sceneSet.addScene(clientScene);
                if (this.progress != null)
                        this.progress.setValue(sceneCounter++);


                System.out.format("[RobocupDatabase] loadScene %d in %.2f seconds!\n", sceneCounter, (float)(System.currentTimeMillis() - tstart) / 1000.0);

                Thread.sleep(CPU_SAVING_DELAY);

            }
        } catch (SQLException ex) {
            ex.printStackTrace();
        }
        catch (InterruptedException ie) {
            ie.printStackTrace();
        }

    }

    void makeSceneSetBatch()
    {
        sceneSet.clear();

     if (this.progress != null) {
         try {
             Statement stm = connection.createStatement();
             ResultSet rs = stm.executeQuery("SELECT * FROM game WHERE id = " + gameID);

             if (rs.next())
                 this.progress.setMaximum(rs.getInt("scene_number") - 1);

             rs.close();
             stm.close();
         } catch (SQLException ex) {
             ex.printStackTrace();
         }
         this.progress.setMinimum(0);
         this.progress.setValue(0);
     }
     int sceneCounter = 0;


     try {
         sstm.setInt(1, gameID);
         ResultSet rs = sstm.executeQuery();

         while (rs.next())
         {
             long tstart = System.currentTimeMillis();

             ClientScene clientScene = new ClientScene();

             int sceneID = rs.getInt("id");
/*
             clientScene.pmode.pmode = rs.getInt("play_mode");
             clientScene.time = rs.getInt("time");

             clientScene.ball.pos.setX(rs.getFloat("ball_pos_x"));
             clientScene.ball.pos.setY(rs.getFloat("ball_pos_y"));
             clientScene.ball.vel.setX(rs.getFloat("ball_vel_x"));
             clientScene.ball.vel.setY(rs.getFloat("ball_vel_y"));
             clientScene.ball.acc.setX(rs.getFloat("ball_accX"));
             clientScene.ball.acc.setY(rs.getFloat("ball_accY"));
             clientScene.ball.mode = rs.getInt("ball_mode");

//                clientScene.ball.bestPos.setX(rs.getFloat("ball_bestPosX"));
//                clientScene.ball.bestPos.setY(rs.getFloat("ball_bestPosY"));

             tstm.setInt(1, sceneID);
             ResultSet trs = tstm.executeQuery();
             if (trs.next())
             {
                 clientScene.left.name = trs.getString("name");
                 clientScene.left.offside = trs.getBoolean("offside");
                 clientScene.left.offsideline = trs.getFloat("offside_line");
             }
             if (trs.next())
             {
                 clientScene.right.name = trs.getString("name");
                 clientScene.right.offside = trs.getBoolean("offside");
                 clientScene.right.offsideline = trs.getFloat("offside_line");
             }
             trs.close();
*/

             pstm.setInt(1, sceneID);
             ResultSet prs = pstm.executeQuery();
             for (int i = 0; i < Param.MAX_PLAYER * 2; i++) {
                 if (!prs.next())
                     break;
                 // int playerID = prs.getInt("id");
/*
                 clientScene.player[i].unum = prs.getInt("unum");
                 clientScene.player[i].pos.setX(prs.getFloat("pos_x"));
                 clientScene.player[i].pos.setY(prs.getFloat("pos_y"));
                 clientScene.player[i].vel.setX(prs.getFloat("vel_x"));
                 clientScene.player[i].vel.setY(prs.getFloat("vel_y"));
                 clientScene.player[i].acc.setX(prs.getFloat("acc_x"));
                 clientScene.player[i].acc.setY(prs.getFloat("acc_y"));
                 clientScene.player[i].side = prs.getInt("side");
                 clientScene.player[i].stamina = prs.getInt("stamina");
                 clientScene.player[i].angle = prs.getFloat("angle");
                 clientScene.player[i].angleNeck = prs.getInt("angle_neck");
                 clientScene.player[i].angleVisible = prs.getInt("angle_visible");
                 clientScene.player[i].viewQuality = prs.getInt("view_quality");
                 clientScene.player[i].sayMessage = prs.getString("say_message");
                 clientScene.player[i].catchDir = prs.getFloat("catch_dir");
                 clientScene.player[i].offside = prs.getBoolean("offside");
                 clientScene.player[i].mode = prs.getInt("mode");
                 clientScene.player[i].type = prs.getInt("type");
                 clientScene.player[i].playerType = prs.getInt("player_type");
                 clientScene.player[i].interceptionTime = prs.getInt("interception_time");
                 clientScene.player[i].roleNumber = prs.getInt("role_number");
                 clientScene.player[i].kickCount = prs.getInt("kick_count");
                 clientScene.player[i].dashCount = prs.getInt("dash_count");
                 clientScene.player[i].turnCount = prs.getInt("turn_count");
                 clientScene.player[i].sayCount = prs.getInt("say_count");
                 clientScene.player[i].turnNeckCount = prs.getInt("turn_neck_count");
                 clientScene.player[i].catchCount = prs.getInt("catch_count");
                 clientScene.player[i].moveCount = prs.getInt("move_count");
                 clientScene.player[i].changeViewCount = prs.getInt("change_view_count");
                 /**
                   @todo hearMessage
                 clientScene.player[i].hearMessage.toString()
                  */
//                 clientScene.player[i].state = prs.getString("state");
  //               clientScene.player[i].reliable = prs.getBoolean("reliable");



/*                    spstm.setInt(1, pwmid);
                 ResultSet sprs = spstm.executeQuery();
                 sprs.close();
*/
//                    while (sprs.next());

             }


             pwmstm.setInt(1, 1);
             ResultSet pwmrs = pwmstm.executeQuery();

             while (pwmrs.next())
             {

                 PlayerWorldModel pwm = new PlayerWorldModel();
 /*                pwm.positionX = pwmrs.getFloat("positionX");
                 pwm.positionY = pwmrs.getFloat("positionY");
                 pwm.positionIsReliable = pwmrs.getBoolean("positionIsReliable");
                 pwm.isGoalie = pwmrs.getBoolean("isGoalie");
                 pwm.simpleBody = pwmrs.getFloat("simpleBody");
                 pwm.speedX = pwmrs.getFloat("speedX");
                 pwm.speedY = pwmrs.getFloat("speedY");
                 pwm.ballInterceptionTime = pwmrs.getInt("ballInterceptionTime");
                 pwm.stamina = pwmrs.getInt("stamina");
                 pwm.playerType = pwmrs.getInt("playerType");
             //                    pwm.lastState = pwmrs.getInt("lastState");
                 pwm.roleNumber = pwmrs.getInt("roleNumber");
             //                    pwm.bestPos.getX() = pwmrs.getFloat("");
             //                    pwm.bestPos.getY() = pwmrs.getFloat("");

                 pwm.neckAngle = pwmrs.getFloat("neckAngle");
                 pwm.bodyAngle = pwmrs.getFloat("bodyAngle");
                 pwm.ballPositionX = pwmrs.getFloat("ballPositionX");
                 pwm.ballPositionY = pwmrs.getFloat("ballPositionY");
                 pwm.ballSpeedX = pwmrs.getFloat("ballSpeedX");
                 pwm.ballSpeedY = pwmrs.getFloat("ballSpeedY");
                 pwm.ballInterceptX = pwmrs.getFloat("ballInterceptX");
                 pwm.ballInterceptY = pwmrs.getFloat("ballInterceptY");
                 pwm.oppNum = pwmrs.getInt("oppNum");
                 pwm.markAreaX = pwmrs.getFloat("markAreaX");
                 pwm.markAreaY = pwmrs.getFloat("markAreaY");
                 pwm.markAreaRad = pwmrs.getFloat("markAreaRad");
                 pwm.shouldMark = pwmrs.getBoolean("shouldMark");
                 pwm.oppInterceptX = pwmrs.getFloat("oppInterceptX");
                 pwm.oppInterceptY = pwmrs.getFloat("oppInterceptY");
                 pwm.offsideY = pwmrs.getFloat("offsideY");
                 pwm.doneAPass = pwmrs.getBoolean("doneAPass");
                 pwm.passToPlayer = pwmrs.getFloat("passToPlayer");
                 pwm.holes = pwmrs.getInt("holes");
                 pwm.based_on_vi = pwmrs.getBoolean("based_on_vi");
             //                    pwm.stateInfos.toString() = pwmrs.getFloat(""); //
 */
                 clientScene.player[1].setWorldModel(pwm);
             }
/*             spstm.setInt(1, 1);
             ResultSet sprs = spstm.executeQuery();
             while (sprs.next());
*/

                 sceneSet.addScene(clientScene);
             if (this.progress != null)
                     this.progress.setValue(sceneCounter++);


             System.out.format("[RobocupDatabase] loadScene %d in %.2f seconds!\n", sceneCounter, (float)(System.currentTimeMillis() - tstart) / 1000.0);


         }
     } catch (SQLException ex) {
         ex.printStackTrace();
     }

    }

    public ArrayList <GameItem> getGameInfos()
    {
        ArrayList<GameItem> result = new ArrayList <GameItem>();

        try {
            Statement stm = connection.createStatement();
            ResultSet rs = stm.executeQuery("SELECT * FROM game");

            while (rs.next())
                result.add(new GameItem(rs.getInt("id"), String.format("%s vs. %s - %d:%d - %d Scenes - %s", rs.getString("team_left"), rs.getString("team_right"), rs.getInt("score_left"), rs.getInt("score_right"), rs.getInt("scene_number"), rs.getTimestamp("date").toString())));

            rs.close();
            stm.close();
        } catch (SQLException ex) {
            ex.printStackTrace();
        }

        return result;
    }

    /**
     * Schliet alle Verbindungen mit der Datenbank
     */
    public void dispose()
    {
         try {
             sstm.close();
             tstm.close();

             pstm.close();

             pwmstm.close();

             spstm.close();

             connection.close();

             connection = null;
             dbReader = null;
         } catch (SQLException ex) {
             ex.printStackTrace();
         }

         System.out.println("[RobocupDatabase] Database disconnected");

    }

    void deleteGame(int gameID)
    {

    }

    public void run() {
        try {
            setName("DAI-NAMITE SoccerScope DatabaseReader");
        } catch (SecurityException e) {
            System.err.println("[RobocupDatabase] Could not set the thread name");
        }

        makeSceneSet();

        dispose();
    }

    private void test() {
        int sceneCounter = 2;
        gameID = 1;
        try {
            sstm.setInt(1, gameID);
            ResultSet rs = sstm.executeQuery();

            while (rs.next()) {
                long start = System.currentTimeMillis();
                int i = 0;
                long tstart = System.currentTimeMillis();
                tstm.setInt(1, sceneCounter);
                ResultSet trs = tstm.executeQuery();
                while (trs.next()) i++;
                System.out.format("[RobocupDatabase] Team %d in %.2f seconds!\n", i++,
                                  (float) (System.currentTimeMillis() - tstart) / 1000.0);

                long pstart = System.currentTimeMillis();
                pstm.setInt(1, 2);
                ResultSet prs = pstm.executeQuery();
                i = 0;
                while (prs.next()) i++;
                System.out.format("[RobocupDatabase] Player %d in %.2f seconds!\n", i++,
                                  (float) (System.currentTimeMillis() - pstart) / 1000.0);

                long pwmstart = System.currentTimeMillis();
                pwmstm.setInt(1, 1);
                ResultSet pwmrs = pwmstm.executeQuery();
                i = 0;
                while (pwmrs.next()) i++;
                System.out.format("[RobocupDatabase] PlayerWorldModel %d in %.2f seconds!\n", i++,
                                  (float) (System.currentTimeMillis() - pwmstart) / 1000.0);

                long spstart = System.currentTimeMillis();
                spstm.setInt(1, 1);
                ResultSet sprs = spstm.executeQuery();
                i = 0;
                while (sprs.next()) i++;
                    System.out.format("[RobocupDatabase] SimplePlayer %d in %.2f seconds!\n", i++,
                       (float) (System.currentTimeMillis() - spstart) / 1000.0);

     System.out.format("[RobocupDatabase] loadScene %d in %.2f seconds!\n", sceneCounter++,
                       (float) (System.currentTimeMillis() - start) / 1000.0);
     System.out.println("=======================================");
            }

       } catch (SQLException ex) {
            ex.printStackTrace();
       }
    }

    public static void main(String[] args) {
        DatabaseReader databasereader = new DatabaseReader(new SceneSet(ClientScene.class));

             databasereader.test();

//        databasereader.makeSceneSet();




    }


}
