/*LICENSE_BEGIN
 * ******************************************************************
 * All original parts of SoccerScope2003 have copyright (2003) by
 * Team YowAI, represented by Mr. Shuhei Shiota
 * The Department of Creative Informatics
 * The Graduate School of Information Science and Technology
 * The University of Tokyo
 * 
 * Additional classes (except libraries in /lib) were developed by 
 * the Dainamite Robocup-Team of the DAI-Labor, TU-Berlin 
 * (www.dai-labor.de)
 * 
 * Extended/new classes/packages are:
 *   soccerscope.analyzer.LogfileAnalyzer
 *   soccerscope.daistatistic.*
 *   soccerscope.daistatistic.charts.*
 *   soccerscope.daistatistic.tools.*
 *   soccerscope.db.*
 *   soccerscope.dialog.*
 *   soccerscope.model.PlayerWorldModel
 *   soccerscope.model.SimplePlayer
 *   soccerscope.net.AgentConnector
 *   soccerscope.training.*
 *   soccerscope.training.conditions.*
 * ******************************************************************
LICENSE_END*/
package soccerscope.model;

import java.io.IOException;
import java.util.StringTokenizer;

import soccerscope.util.Time;

public abstract class SceneBuilder {
	public final static int NO_SOURCE = 0x000;

	public final static int SERVER = 0x001;

	public final static int MONITOR_PROTOCOL_1 = 0x002;

	public final static int MONITOR_PROTOCOL_2 = 0x004;

	public final static int LOGFILE = 0x010;

	public final static int LOGFILE_VER_1 = 0x020;

	public final static int LOGFILE_VER_2 = 0x040;

	public final static int LOGFILE_VER_3 = 0x080;

	protected int type;

	public int getType() {
		return this.type;
	}

	public final static int NO_INFO = 0;

	public final static int SHOW_MODE = 1;

	public final static int MSG_MODE = 2;

	public final static int DRAW_MODE = 3;

	public final static int BLANK_MODE = 4;

	public final static int PM_MODE = 5;

	public final static int TEAM_MODE = 6;

	public final static int PT_MODE = 7;

	public final static int PARAM_MODE = 8;

	public final static int PPARAM_MODE = 9;

	public int getPacketType(byte[] packet) {
		return readShort(packet, 0);
	};

	public final static float SHOWINFO_SCALE = 16.0f;

	public final static float SHOWINFO_SCALE2 = 65536.0f;

	abstract public int readPacket(byte[] packet) throws IOException;

	abstract public ClientScene makeScene(byte[] packet, PlayMode playmode,
			Team left, Team right);

	public PlayMode makePlayMode(byte[] packet) {
		PlayMode playmode = new PlayMode();

		// pmode (char)
		playmode.pmode = readChar(packet, 2);
		return playmode;
	}

	public Team makeLeftTeam(byte[] packet, int offset) {
		Team team = new Team(Team.LEFT_SIDE);

		int i;
		for (i = 0; i < 16; i++)
			if (readChar(packet, i + offset) == 0)
				break;
		team.name = new String(packet, offset, i);
		offset += 16;
		team.score = readShort(packet, offset);
		offset += 2;
		return team;
	}

	public Team makeRightTeam(byte[] packet, int offset) {
		Team team = new Team(Team.RIGHT_SIDE);

		offset += 16;
		offset += 2;

		int i;
		for (i = 0; i < 16; i++)
			if (readChar(packet, i + offset) == 0)
				break;
		team.name = new String(packet, offset, i);
		offset += 16;
		team.score = readShort(packet, offset);
		offset += 2;
		return team;
	}

	public void addMessage(ClientScene clientScene, String message) {
		// kick-offϼΤ
		if (clientScene.time <= 0 || clientScene.time == Time.MAX_TIME) {
			return;
		}

		// System.out.println(clientScene.time + " " + message);

		// header sender commandڤʬ
		StringTokenizer messageTokenizer = new StringTokenizer(message, " ");
		String header = messageTokenizer.nextToken();
		if (header.compareTo("Recv") != 0)
			return;
		String sender = messageTokenizer.nextToken();
		StringBuffer command = new StringBuffer(messageTokenizer.nextToken());
		while (messageTokenizer.hasMoreTokens()) {
			command.append(" " + messageTokenizer.nextToken());
		}

		// System.out.println("header: " + header);
		// System.out.println("sender: " + sender);
		// System.out.println("command: " + command);

		int lastindex = sender.lastIndexOf(':');
		String coach = sender.substring(0, lastindex);
		if (coach.compareTo("Coach") == 0) {
			return;
		}

		// sender 
		int endindex = sender.lastIndexOf('_');
		String teamname = sender.substring(0, endindex);
		sender = sender.replace(':', ' ');
		sender = sender.replace('_', ' ');
		sender = sender.trim();
		int unum;
		try {
			unum = Integer.parseInt(sender.substring(endindex + 1));
		} catch (NumberFormatException ne) {
			return;
		}
		Player player;
		if (teamname.compareTo(clientScene.left.name) == 0) {
			player = clientScene.player[unum - 1];
		} else {
			player = clientScene.player[unum - 1 + Param.MAX_PLAYER];
		}

		// command 
		String commands = command.toString();
		commands = commands.replaceAll("\\)\\(", "\\) \\(");
		StringTokenizer commandTokenizer = new StringTokenizer(commands, " ");
		while (commandTokenizer.hasMoreTokens()) {
			String commandType = commandTokenizer.nextToken();
			String arg1 = null, arg2 = null;
			if (commandType.compareTo("(say") == 0) {
				StringBuffer messages = new StringBuffer(commandTokenizer
						.nextToken());
				while (commandTokenizer.hasMoreTokens())
					messages.append(" " + commandTokenizer.nextToken());
				arg1 = messages.substring(0, messages.length() - 1);
				// arg1 = command.substring(new String("(say ").length(),
				// command.length() -1);
				player.sayMessage = arg1;
				// System.out.println("say :" + arg1);

			} else if (commandType.compareTo("(dash") == 0) {
				// ޥɰν
				arg1 = commandTokenizer.nextToken();
				arg1 = arg1.replace(')', ' ');
				arg1 = arg1.trim();
				try {
					player.acc = player.getDashAccelerate(Float
							.parseFloat(arg1));
				} catch (NumberFormatException ne) {
					return;
				}

			} else if (commandType.compareTo("(kick") == 0) {
				// ޥɰν
				arg1 = commandTokenizer.nextToken();
				arg1 = arg1.replace(')', ' ');
				arg1 = arg1.trim();
				if (commandTokenizer.hasMoreTokens()) {
					arg2 = commandTokenizer.nextToken();
					arg2 = arg2.replace(')', ' ');
					arg2 = arg2.trim();
				}
				// åˤ®٤η׻
				if (player.isKickable(clientScene.ball.pos)) {
					try {
						clientScene.ball.acc.add(player.getKickAccelerate(
								clientScene.ball.pos, Float.parseFloat(arg1),
								Float.parseFloat(arg2)));
						if (clientScene.ball.acc.length() > Param.BALL_ACCEL_MAX) {
							clientScene.ball.acc = clientScene.ball.acc
									.normalize();
							clientScene.ball.acc.scale(Param.BALL_ACCEL_MAX);
						}
					} catch (NumberFormatException ne) {
						return;
					}
				}

			} else if (commandType.compareTo("(catch") == 0) {
				// ޥɰν
				arg1 = commandTokenizer.nextToken();
				arg1 = arg1.replace(')', ' ');
				arg1 = arg1.trim();
				try {
					player.catchDir = Float.parseFloat(arg1);
				} catch (NumberFormatException ne) {
					return;
				}

			} else if (commandType.compareTo("(turn") == 0) {

			} else if (commandType.compareTo("(turn_neck") == 0) {

			} else if (commandType.compareTo("(tackle") == 0) {

			} else if (commandType.compareTo("(attentionto") == 0) {

			} else {

			}
		}
	}

	public void makeHeteroParam(byte[] packet) {
		int offset = 0;

		offset += 2;
		int id = 0;
		if ((this.type & LOGFILE) == LOGFILE) {
			id = readShort(packet, offset);
			offset += 2;
			offset += 2; // padding
		} else if ((this.type & SERVER) == SERVER) {
			offset += 2; // padding
			id = readShort(packet, offset);
			offset += 2;
			offset += 2; // padding
		}
		double player_speed_max = readLong(packet, offset) / 65535.0;
		offset += 4;
		double stamina_inc_max = readLong(packet, offset) / 65535.0;
		offset += 4;
		double player_decay = readLong(packet, offset) / 65535.0;
		offset += 4;
		double inertia_moment = readLong(packet, offset) / 65535.0;
		offset += 4;
		double dash_power_rate = readLong(packet, offset) / 65535.0;
		offset += 4;
		double player_size = readLong(packet, offset) / 65535.0;
		offset += 4;
		double kickable_margin = readLong(packet, offset) / 65535.0;
		offset += 4;
		double kick_rand = readLong(packet, offset) / 65535.0;
		offset += 4;
		double extra_stamina = readLong(packet, offset) / 65535.0;
		offset += 4;
		double effort_max = readLong(packet, offset) / 65535.0;
		offset += 4;
		double effort_min = readLong(packet, offset) / 65535.0;
		offset += 4;

		HeteroParam.put(id, new HeteroParam((float) player_speed_max,
				(float) stamina_inc_max, (float) player_decay,
				(float) inertia_moment, (float) dash_power_rate,
				(float) player_size, (float) kickable_margin,
				(float) kick_rand, (float) extra_stamina, (float) effort_max,
				(float) effort_min));

	}

	public void printServerParameter(byte[] packet) {

	}

	public void printPlayerParameter(byte[] packet) {

	}

	// 4byteɤ߹
	protected long readLong(byte[] buf, int offset) {
		return ((buf[offset] & 0xff) << 24) | ((buf[offset + 1] & 0xff) << 16)
				| ((buf[offset + 2] & 0xff) << 8) | ((buf[offset + 3] & 0xff));
	}

	// 2byteɤ߹
	protected int readShort(byte[] buf, int offset) {
		return (short) ((buf[offset] & 0xff) << 8) | (buf[offset + 1] & 0xff);
	}

	// 2byte̵ɤ߹
	protected int readUnsignedShort(byte[] buf, int offset) {
		return ((buf[offset] & 0xff) << 8) | (buf[offset + 1] & 0xff);
	}

	// 1byteɤ߹
	protected int readChar(byte[] buf, int offset) {
		return (buf[offset] & 0xff);
	}
}
