/*LICENSE_BEGIN
 * ******************************************************************
 * All original parts of SoccerScope2003 have copyright (2003) by
 * Team YowAI, represented by Mr. Shuhei Shiota
 * The Department of Creative Informatics
 * The Graduate School of Information Science and Technology
 * The University of Tokyo
 * 
 * Additional classes (except libraries in /lib) were developed by 
 * the Dainamite Robocup-Team of the DAI-Labor, TU-Berlin 
 * (www.dai-labor.de)
 * 
 * Extended/new classes/packages are:
 *   soccerscope.analyzer.LogfileAnalyzer
 *   soccerscope.daistatistic.*
 *   soccerscope.daistatistic.charts.*
 *   soccerscope.daistatistic.tools.*
 *   soccerscope.db.*
 *   soccerscope.dialog.*
 *   soccerscope.model.PlayerWorldModel
 *   soccerscope.model.SimplePlayer
 *   soccerscope.net.AgentConnector
 *   soccerscope.training.*
 *   soccerscope.training.conditions.*
 * ******************************************************************
LICENSE_END*/
package soccerscope.util.analyze;

import java.util.ArrayList;

import soccerscope.model.ClientScene;
import soccerscope.model.GameEvent;
import soccerscope.model.Param;
import soccerscope.model.PlayMode;
import soccerscope.model.Player;

public class DribbleAnalyzer extends SceneAnalyzer {

	public static String NAME = "Dribble";

	private ArrayList<Dribbler> dribbleLeftList = new ArrayList<Dribbler>();

	private ArrayList<Dribbler> dribbleRightList = new ArrayList<Dribbler>();

	private Dribbler leftDribbler;

	private Dribbler rightDribbler;

	@Override
	public void init() {
		super.init();
		this.dribbleLeftList.clear();
		this.dribbleRightList.clear();
		this.leftDribbler = null;
		this.rightDribbler = null;
	}

	@Override
	public String getName() {
		return NAME;
	}

	public class Dribbler {
		public int time;

		public Player player;

		public boolean counted;

		public Dribbler() {
			this.time = 0;
			this.player = null;
			this.counted = false;
		}

		public Dribbler(int time, Player player) {
			this.time = time;
			this.player = player;
			this.counted = false;
		}

		public void set(Dribbler k) {
			this.time = k.time;
			this.player = k.player;
			this.counted = k.counted;
		}

		@Override
		public String toString() {
			return "dribbler ::" + this.time + ": " + this.player.unum;
		}
	}

	@Override
	public GameEvent analyze(ClientScene clientScene, ClientScene prev) {

		if (prev == null)
			return null;

		// ץ쥤ʳ, 
		if (clientScene.pmode.pmode != PlayMode.PM_PlayOn) {
			this.leftDribbler = null;
			this.rightDribbler = null;
			return null;
		}

		Dribbler left = null;
		Dribbler right = null;

		for (int i = 0; i < Param.MAX_PLAYER; i++) {
			if ((clientScene.player[i].isKicking() && prev.player[i]
					.isKickable(prev.ball.pos))
					|| (clientScene.player[i].isCatching() && prev.player[i]
							.isCatchable(prev.ball.pos))
					|| (clientScene.player[i].isTackling() && prev.player[i]
							.canTackle(prev.ball.pos))) {
				left = new Dribbler(clientScene.time, clientScene.player[i]);
			}
		}
		for (int i = Param.MAX_PLAYER; i < Param.MAX_PLAYER * 2; i++) {
			if ((clientScene.player[i].isKicking() && prev.player[i]
					.isKickable(prev.ball.pos))
					|| (clientScene.player[i].isCatching() && prev.player[i]
							.isCatchable(prev.ball.pos))
					|| (clientScene.player[i].isTackling() && prev.player[i]
							.canTackle(prev.ball.pos))) {
				right = new Dribbler(clientScene.time, clientScene.player[i]);
			}
		}

		if (left != null && right != null) {
			this.leftDribbler = null;
			this.rightDribbler = null;
			left = null;
			right = null;
		}
		if (left != null) {
			if (this.leftDribbler != null
					&& this.leftDribbler.player.unum == left.player.unum
					&& !this.leftDribbler.counted
					&& doDashed(left.player, this.leftDribbler.player)) {
				countUpLeft(this.leftDribbler.time);
				this.dribbleLeftList.add(this.leftDribbler);
				this.leftDribbler.counted = true;
				left.counted = true;
			}
			if (this.leftDribbler != null
					&& this.leftDribbler.player.unum == left.player.unum
					&& this.leftDribbler.counted)
				left.counted = true;
			this.leftDribbler = left;
			if (right == null)
				this.rightDribbler = null;
		}
		if (right != null) {
			if (this.rightDribbler != null
					&& this.rightDribbler.player.unum == right.player.unum
					&& !this.rightDribbler.counted
					&& doDashed(right.player, this.rightDribbler.player)) {
				countUpRight(this.rightDribbler.time);
				this.dribbleRightList.add(this.rightDribbler);
				this.rightDribbler.counted = true;
				right.counted = true;
			}
			if (this.rightDribbler != null
					&& this.rightDribbler.player.unum == right.player.unum
					&& this.rightDribbler.counted)
				right.counted = true;
			this.rightDribbler = right;
			if (left == null)
				this.leftDribbler = null;
		}

		return null;
	}
}
