/*LICENSE_BEGIN
 * ******************************************************************
 * All original parts of SoccerScope2003 have copyright (2003) by
 * Team YowAI, represented by Mr. Shuhei Shiota
 * The Department of Creative Informatics
 * The Graduate School of Information Science and Technology
 * The University of Tokyo
 * 
 * Additional classes (except libraries in /lib) were developed by 
 * the Dainamite Robocup-Team of the DAI-Labor, TU-Berlin 
 * (www.dai-labor.de)
 * 
 * Extended/new classes/packages are:
 *   soccerscope.analyzer.LogfileAnalyzer
 *   soccerscope.daistatistic.*
 *   soccerscope.daistatistic.charts.*
 *   soccerscope.daistatistic.tools.*
 *   soccerscope.db.*
 *   soccerscope.dialog.*
 *   soccerscope.model.PlayerWorldModel
 *   soccerscope.model.SimplePlayer
 *   soccerscope.net.AgentConnector
 *   soccerscope.training.*
 *   soccerscope.training.conditions.*
 * ******************************************************************
LICENSE_END*/
package soccerscope.util.analyze;

import java.text.NumberFormat;
import java.util.Iterator;

import soccerscope.model.ClientScene;
import soccerscope.model.GameEvent;
import soccerscope.model.Param;
import soccerscope.model.WorldModel;
import soccerscope.util.geom.Point2f;

public class PassDistAnalyzer extends PassAnalyzer {

	public static String NAME = "Pass Dist";

	private float leftMin;

	private float leftAve;

	private float leftMax;

	private float rightMin;

	private float rightAve;

	private float rightMax;

	@Override
	public void init() {
		super.init();
	}

	@Override
	public String getName() {
		return NAME;
	}

	@Override
	public Object getValueAt(int col, int fromTime, int toTime) {
		count(fromTime, toTime);
		NumberFormat nf = NumberFormat.getInstance();
		nf.setMaximumIntegerDigits(2);
		nf.setMaximumFractionDigits(2);
		switch (col) {
		case ROW_NAME:
			return getName();
		case LEFT:
			if (this.lcount == -1)
				return "--";
			// return "(" + nf.format(leftMin) + " : " + nf.format(leftAve /
			// lcount) +
			// " : " + nf.format(leftMax) + ")";
			return nf.format(this.leftAve / this.lcount);
		case RIGHT:
			if (this.rcount == -1)
				return "--";
			// return "(" + nf.format(rightMin) + " : " + nf.format(rightAve /
			// rcount) +
			// " : " + nf.format(rightMax) + ")";
			return nf.format(this.rightAve / this.rcount);
		default:
			return " ";
		}
	}

	@Override
	public void count(int fromTime, int toTime) {
		Iterator it = PassAnalyzer.getPassLeftList().iterator();
		this.lcount = 0;
		this.leftAve = 0;
		this.leftMin = 100;
		this.leftMax = -100;
		while (it.hasNext()) {
			Pass pass = (Pass) it.next();
			if (fromTime <= pass.sender.time && pass.receiver.time <= toTime) {
				ClientScene rscene = (ClientScene) WorldModel.getInstance()
						.getSceneSet().getScene(pass.receiver.time);
				ClientScene sscene = (ClientScene) WorldModel.getInstance()
						.getSceneSet().getScene(pass.sender.time);
				Point2f spos = sscene.player[pass.sender.unum - 1].pos;
				Point2f rpos = rscene.player[pass.receiver.unum - 1].pos;
				float dist = spos.dist(rpos);
				this.leftAve += dist;
				this.lcount++;
				if (this.leftMin > dist)
					this.leftMin = dist;
				if (this.leftMax < dist)
					this.leftMax = dist;
			}
		}
		if (this.lcount == 0)
			this.lcount = -1;

		it = PassAnalyzer.getPassRightList().iterator();
		this.rcount = 0;
		this.rightAve = 0;
		this.rightMin = 100;
		this.rightMax = -100;
		while (it.hasNext()) {
			Pass pass = (Pass) it.next();
			if (fromTime <= pass.sender.time && pass.receiver.time <= toTime) {
				ClientScene rscene = (ClientScene) WorldModel.getInstance()
						.getSceneSet().getScene(pass.receiver.time);
				ClientScene sscene = (ClientScene) WorldModel.getInstance()
						.getSceneSet().getScene(pass.sender.time);
				Point2f spos = sscene.player[pass.sender.unum
						+ Param.MAX_PLAYER - 1].pos;
				Point2f rpos = rscene.player[pass.receiver.unum
						+ Param.MAX_PLAYER - 1].pos;
				float dist = spos.dist(rpos);
				this.rightAve += dist;
				this.rcount++;
				if (this.rightMin > dist)
					this.rightMin = dist;
				if (this.rightMax < dist)
					this.rightMax = dist;
			}
		}
		if (this.rcount == 0)
			this.rcount = -1;
	}

	@Override
	public GameEvent analyze(ClientScene clientScene, ClientScene prev) {
		return null;
	}
}
