/*LICENSE_BEGIN
 * ******************************************************************
 * All original parts of SoccerScope2003 have copyright (2003) by
 * Team YowAI, represented by Mr. Shuhei Shiota
 * The Department of Creative Informatics
 * The Graduate School of Information Science and Technology
 * The University of Tokyo
 * 
 * Additional classes (except libraries in /lib) were developed by 
 * the Dainamite Robocup-Team of the DAI-Labor, TU-Berlin 
 * (www.dai-labor.de)
 * 
 * Extended/new classes/packages are:
 *   soccerscope.analyzer.LogfileAnalyzer
 *   soccerscope.daistatistic.*
 *   soccerscope.daistatistic.charts.*
 *   soccerscope.daistatistic.tools.*
 *   soccerscope.db.*
 *   soccerscope.dialog.*
 *   soccerscope.model.PlayerWorldModel
 *   soccerscope.model.SimplePlayer
 *   soccerscope.net.AgentConnector
 *   soccerscope.training.*
 *   soccerscope.training.conditions.*
 * ******************************************************************
LICENSE_END*/
package soccerscope.view;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.RenderingHints;
import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.util.ArrayList;

import javax.swing.JComponent;

import soccerscope.model.ClientScene;
import soccerscope.model.Param;
import soccerscope.model.SoccerObjectID;
import soccerscope.util.geom.Point2f;
import soccerscope.view.layer.Layer;
import soccerscope.view.layer.PitchLayer;
import soccerscope.view.layer.PitchLineLayer;

public class FieldPane extends JComponent implements SoccerObjectID,
		ScopeWindow {

	private static final long serialVersionUID = 1170641740574844744L;

	/**
	 * public  *
	 */
	public final static int DRAW_ANTIALIAS = 1;

	public final static int DRAW_OPTION_MAX = 2;

	/* ѿ */
	private boolean[] selectedTable;

	private boolean[] offlineSelectedTable;

	private boolean[] drawTable;

	// ɽ륷
	/* ɽ֥ */
	private ClientScene clientScene;

	// private LocalScene agentClientScene; //ClientScene of AgentWorldModel
	private ArrayList[] agentPlan;

	// Ψ
	private float magnify = 6.0f; // ΤγΨ

	private float playerMagnify = 3.6f; // ץ쥤γΨ

	private float ballMagnify = 7.2f; // ܡγΨ

	// ѿ֥
	private Point2f centerF; // (եɺɸ)

	private Point2f centerS; // ̤(̺ɸ)

	private ArrayList<Layer> backgroundList;

	private ArrayList<Layer> foregroundList;

	private ArrayList shapeList;

	public FieldPane(ClientScene clientScene) {
		super();

		this.clientScene = clientScene;
		// agentClientScene = null;

		this.agentPlan = new ArrayList[Param.MAX_PLAYER];
		for (int i = 0; i < Param.MAX_PLAYER; i++)
			this.agentPlan[i] = null;

		this.selectedTable = new boolean[MAX_ID];
		for (int i = 0; i < MAX_ID; i++)
			this.selectedTable[i] = false;

		this.offlineSelectedTable = new boolean[MAX_ID];
		for (int i = 0; i < MAX_ID; i++)
			this.offlineSelectedTable[i] = false;

		this.drawTable = new boolean[DRAW_OPTION_MAX];
		this.drawTable[DRAW_ANTIALIAS] = false;

		this.centerF = new Point2f(0.0f, 0.0f);
		this.centerS = new Point2f();

		this.backgroundList = new ArrayList<Layer>();
		this.foregroundList = new ArrayList<Layer>();
		this.shapeList = null;

		this.backgroundList.add(new PitchLayer(this));
		this.foregroundList.add(new PitchLineLayer(this));

	}

	// ɽ
	public void setScene(ClientScene clientScene) {
		this.clientScene = clientScene;
	}

	// ɽμ
	public ClientScene getScene() {
		return this.clientScene;
	}

	// Called from clientScenetScenetScenetScene player sets the current
	// clientScenetScenetScenetScene
	/*
	 * public void setAgentScene(LocalScene localScene) { agentClientScene =
	 * localScene; } // ȥɤμ public LocalScene
	 * getAgentScene() { return agentClientScene; }
	 */
	// ȥץ
	public void setAgentPlan(ArrayList list, int index) {
		this.agentPlan[index] = list;
	}

	// ȥץμ
	public ArrayList getAgentPlan(int index) {
		return this.agentPlan[index];
	}

	// ȥץ˴
	public void clearAgentPlan(int index) {
		this.agentPlan[index] = null;
	}

	// Хåɥ쥤䡼ɲ
	public void addBackgroundLayer(Layer layer) {
		this.backgroundList.add(layer);
	}

	// եɥ쥤䡼ɲ
	public void addForegroundLayer(Layer layer) {
		this.foregroundList.add(layer);
	}

	public ArrayList getBackgroundList() {
		return this.backgroundList;
	}

	public ArrayList getForegroundList() {
		return this.foregroundList;
	}

	// 褹޷ꥹȤ
	public void setShapeList(ArrayList list) {
		this.shapeList = list;
		repaint();
	}

	// 褹޷ꥹȤõ
	public void clearShapeList() {
		if (this.shapeList != null) {
			this.shapeList.clear();
			this.shapeList = null;
			repaint();
		}
	}

	public void setSelected(int id, boolean flag) {
		this.selectedTable[id] = flag;
		repaint();
	}

	/**
	 * indicates the player with id has a mark
	 * 
	 * @param id
	 * @return
	 */
	public boolean isSelected(int id) {
		return this.selectedTable[id];
	}

	public void setOfflineSelected(int id, boolean flag) {
		this.offlineSelectedTable[id] = flag;
		repaint();
	}

	public boolean isOfflineSelected(int id) {
		if (id > BALL || id < 0)
			return false;
		return this.offlineSelectedTable[id];
	}

	// 襪ץ
	public void setDrawOption(int key, boolean flag) {
		if (key < DRAW_OPTION_MAX)
			this.drawTable[key] = flag;
		repaint();
	}

	// 襪ץμ
	public boolean getDrawOption(int key) {
		if (key < DRAW_OPTION_MAX)
			return this.drawTable[key];
		return false;
	}

	// Ψμ
	public float getMagnify() {
		return this.magnify;
	}

	// ץ쥤䡼γΨμ
	public float getPlayerMagnify() {
		return this.playerMagnify;
	}

	// ܡγΨμ
	public float getBallMagnify() {
		return this.ballMagnify;
	}

	// Ψꤹ
	// ץ쥤ȥܡγΨϼưǷ׻
	public void setMagnify(float magnify) {
		this.magnify = Math.max(1, magnify);
		float playerWidth = Param.PLAYER_SIZE * this.magnify;
		float ballWidth = Param.BALL_SIZE * this.magnify;
		// System.out.println(playerWidth + " " + ballWidth);
		this.playerMagnify = Math.max(1, 7 / playerWidth);
		this.ballMagnify = Math.max(1, 3.5f / ballWidth);
		// System.out.println(playerMagnify + " " + ballMagnify);
		repaint();
	}

	// Ψꤹ
	public void setMagnify(float magnify, float ballMagnify, float playerMagnify) {
		this.magnify = Math.max(1, magnify);
		this.ballMagnify = Math.max(1, ballMagnify);
		this.playerMagnify = Math.max(1, playerMagnify);
		repaint();
	}

	// եΤ̥ˤޤ褦˳Ψꤹ
	public void fitMagnify() {
		Dimension d = this.getSize();
		int width = (int) (Param.PITCH_LENGTH + 0.2 * Param.PITCH_MARGIN);
		int height = (int) (Param.PITCH_WIDTH + 0.2 * Param.PITCH_MARGIN);
		this.magnify = Math.min(d.width / width, d.height / height);
		setMagnify(this.magnify);
		repaint();
	}

	// եΤꥵˤޤ褦˳Ψꤹ
	public void fitMagnify(Dimension d) {
		int width = (int) (Param.PITCH_LENGTH + 0.2 * Param.PITCH_MARGIN);
		int height = (int) (Param.PITCH_WIDTH + 0.2 * Param.PITCH_MARGIN);
		this.magnify = Math.min(d.width / width, d.height / height);
		setMagnify(this.magnify);
		repaint();
	}

	public void setPlayerMagnify(float playerMagnify) {
		this.playerMagnify = playerMagnify;
		repaint();
	}

	public void setBallMagnify(float ballMagnify) {
		this.ballMagnify = ballMagnify;
		repaint();
	}

	// եɺɸϤؤΥեѴ
	public AffineTransform getAffineTransform() {
		return new AffineTransform(this.magnify, 0, 0, this.magnify,
				-this.centerF.x * this.magnify + this.centerS.x,
				-this.centerF.y * this.magnify + this.centerS.y);
	}

	// եɺɸ̺ɸѴ
	public Point2f fieldToScreen(Point2f p) {
		Point2f p1 = new Point2f(p);
		p1.sub(this.centerF);
		p1.scale(this.magnify);
		p1.add(this.centerS);
		return p1;
	}

	// եɺɸ̺ɸѴ
	// 侩Ƥޤ(LayerdrawRectʤɤΤ߻)
	public int F2SX(float fx) {
		return (int) ((fx - this.centerF.x) * this.magnify + this.centerS.x);
	}

	public int F2SY(float fy) {
		return (int) ((fy - this.centerF.y) * this.magnify + this.centerS.y);
	}

	public int F2SSX(float fx) {
		return (int) (fx * this.magnify);
	}

	public int F2SSY(float fy) {
		return (int) (fy * this.magnify);
	}

	// ̺ɸեɺɸѴ
	public Point2f screenToField(Point2f p) {
		Point2f p1 = new Point2f(p);
		p1.sub(this.centerS);
		p1.scale(1.0f / this.magnify);
		p1.add(this.centerF);
		return p1;
	}

	// ̺ɸեɺɸѴ
	public Point2f screenToField(Point p) {
		Point2f p1 = new Point2f(p.x, p.y);
		p1.sub(this.centerS);
		p1.scale(1.0f / this.magnify);
		p1.add(this.centerF);
		return p1;
	}

	// (եɺɸ)ꤹ
	public void setWatchPoint(float x, float y) {
		this.centerF.set(x, y);
		repaint();
	}

	// (եɺɸ)ꤹ
	public void setWatchPoint(Point2f p) {
		this.centerF.set(p);
		repaint();
	}

	// (եɺɸ)ꤹ
	public void setWatchPoint(Point p) {
		this.centerF.set(screenToField(p));
		repaint();
	}

	// (եɺɸ)֤
	public Point2f getWatchPoint() {
		return new Point2f(this.centerF);
	}

	// եɺɸϤǤϰϤ򤫤
	public Dimension getDrawableSize() {
		Dimension d = this.getSize();
		int width = (int) (d.width / this.magnify);
		int height = (int) (d.height / this.magnify);
		return new Dimension(width, height);
	}

	@Override
	public Dimension getPreferredSize() {
		int width = (int) ((Param.PITCH_LENGTH + 2.0 * Param.PITCH_MARGIN) * this.magnify);
		int height = (int) ((Param.PITCH_WIDTH + 2.0 * Param.PITCH_MARGIN) * this.magnify);
		return new Dimension(width, height);
	}

	@Override
	public Dimension getMinimumSize() {
		return getPreferredSize();
	}

	// ̤
	@Override
	public void paintComponent(Graphics g) {
		if (this.clientScene == null)
			return;

		Dimension d = this.getSize();
		this.centerS.x = d.width / 2;
		this.centerS.y = d.height / 2;

		Graphics2D g2 = (Graphics2D) g;
		if (this.drawTable[DRAW_ANTIALIAS]) {
			g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
					RenderingHints.VALUE_ANTIALIAS_ON);
		}

		int bs = this.backgroundList.size();
		for (int i = 0; i < bs; i++) {
			// System.out.println((System.currentTimeMillis() - time)
			// + " " + ((Layer)backgroundList.get(i)).getLayerName());
			((Layer) this.backgroundList.get(i)).draw(g2);
		}

		int fs = this.foregroundList.size();
		for (int i = 0; i < fs; i++) {
			// System.out.println((System.currentTimeMillis() - time)
			// + " " + ((Layer)foregroundList.get(i)).getLayerName());
			((Layer) this.foregroundList.get(i)).draw(g2);
		}
		// System.out.println((System.currentTimeMillis() - time));

		if (this.shapeList != null) {
			g2.setColor(Color.lightGray);
			int sls = this.shapeList.size();
			for (int i = 0; i < sls; i++)
				g2.draw((Shape) this.shapeList.get(i));
		}
	}

}
