/*LICENSE_BEGIN
 * ******************************************************************
 * All original parts of SoccerScope2003 have copyright (2003) by
 * Team YowAI, represented by Mr. Shuhei Shiota
 * The Department of Creative Informatics
 * The Graduate School of Information Science and Technology
 * The University of Tokyo
 * 
 * Additional classes (except libraries in /lib) were developed by 
 * the Dainamite Robocup-Team of the DAI-Labor, TU-Berlin 
 * (www.dai-labor.de)
 * 
 * Extended/new classes/packages are:
 *   soccerscope.analyzer.LogfileAnalyzer
 *   soccerscope.daistatistic.*
 *   soccerscope.daistatistic.charts.*
 *   soccerscope.daistatistic.tools.*
 *   soccerscope.db.*
 *   soccerscope.dialog.*
 *   soccerscope.model.PlayerWorldModel
 *   soccerscope.model.SimplePlayer
 *   soccerscope.net.AgentConnector
 *   soccerscope.training.*
 *   soccerscope.training.conditions.*
 * ******************************************************************
LICENSE_END*/
package soccerscope.view.layer;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.util.ArrayList;
import java.util.List;

import soccerscope.model.ClientScene;
import soccerscope.model.Param;
import soccerscope.model.PlayMode;
import soccerscope.model.Player;
import soccerscope.util.geom.Point2f;
import soccerscope.util.geom.Vector2f;
import soccerscope.view.FieldPane;

public class CommentaryLayer extends Layer {

	public CommentaryLayer(FieldPane fieldPane, boolean enable) {
		super(fieldPane, enable);
		this.ic = new IsCentering();
		this.kickerlist = new PlayerList();
	}

	@Override
	public String getLayerName() {
		return "Commentary";
	}

	// Commentary 
	private int preLastKickerId = -1;

	private int lastKickerId = -1;

	private ClientScene lastKickClientScene;

	private PlayerList kickerlist;

	private IsCentering ic;

	@Override
	public void draw(Graphics g) {
		if (!this.enable)
			return;

		ClientScene clientScene = this.fieldPane.getScene();
		float ballMagnify = this.fieldPane.getBallMagnify();
		float playerMagnify = this.fieldPane.getPlayerMagnify();

		float playerR = Param.PLAYER_SIZE * playerMagnify;
		for (int i = 0; i < Param.MAX_PLAYER * 2; i++) {
			playerR = clientScene.player[i].getSize() * playerMagnify;
			if (!clientScene.player[i].isEnable())
				continue;

			if (clientScene.player[i].isKicking()) {
				this.lastKickerId = i;
				this.kickerlist.add(clientScene.player[i]);
				this.lastKickClientScene = clientScene;
			}
		}

		if ((clientScene.pmode.pmode == PlayMode.PM_PlayOn)
				&& (this.lastKickerId > -1)) {

			// centering
			if (this.ic.isCentering(clientScene.player[this.lastKickerId].pos,
					clientScene)) {
				playerR = clientScene.player[this.lastKickerId].getSize()
						* playerMagnify;
				Point2f drawPoint = new Point2f(
						clientScene.player[this.lastKickerId].pos);
				drawPoint.add(new Point2f(playerR, playerR));
				g.setFont(boldFont);
				g.setColor(Color.cyan);
				drawString(g, "Centering", drawPoint);
			}
			Vector2f playerangle = new Vector2f(
					(float) Math
							.cos((Math.PI / 180)
									* (this.lastKickClientScene.player[this.lastKickerId].angle)),
					(float) Math
							.sin((Math.PI / 180)
									* (this.lastKickClientScene.player[this.lastKickerId].angle)));
			if (this.lastKickClientScene.ball.vel.length() < 0.2f) {
				playerangle = new Vector2f(clientScene.ball.vel);
				// System.out.println("Henkokiu");
			}

			if ((this.lastKickClientScene != null)
					&& this.kickerlist.isDribble()
					&& (clientScene.player[this.lastKickerId].vel.length() > 0.01f)
					&& ((clientScene.player[this.lastKickerId].pos)
							.distance(clientScene.ball.pos) > .3f)
					&& (clientScene.ball.vel.angle(playerangle) < 1.0f)) {
				playerR = clientScene.player[this.lastKickerId].getSize()
						* playerMagnify;
				Point2f drawPoint = new Point2f(
						clientScene.player[this.lastKickerId].pos);
				drawPoint.add(new Point2f(playerR, playerR));
				g.setFont(boldFont);
				g.setColor(clientScene.player[this.lastKickerId].getColor());

				if (this.lastKickClientScene.ball.vel.length() < 1.5f) {
					drawString(g, "Dribble", drawPoint);
				} else {
					// drawString(g, "Pass", drawPoint);
				}
			}
			// System.out.println("player: " +
			// clientScenetScenetScene.player[lastKickerId].vel.length());
			// System.out.println("ball.vel.length: " +
			// clientScenetScenetScene.ball.vel.length());
			// System.out.println("player distance ball: " +
			// (clientScenetScenetScenetScenetScenetScene.player[lastKickerId].pos).distance(clientScenetScenetScenetScenetScenetScene.ball.pos));
			// System.out.println("ball.vel>>>>>>" +
			// clientScenetScenetScene.ball.vel);
			// System.out.println("player.vel>>>>>>" +
			// clientScenetScenetScene.player[lastKickerId].vel.length());
			// System.out.println("ballvel.angle(player.vel)>>>>>>" +
			// clientScenetScenetScene.ball.vel.angle(playerangle));
			// System.out.println("");

			// System.out.println(clientScenetScenetScenetScenetScenetScene.player[lastKickerId].pos.distance(clientScenetScenetScenetScenetScenetScene.ball.pos));
			// drawRectByPoint(g,
			// clientScenetScenetScenetScenetScenetScene.player[lastKickerId].pos,
			// clientScenetScenetScenetScenetScenetScene.ball.pos);
		}
		g.setFont(plainFont);
	}

	private final static Font boldFont = new Font("", Font.BOLD, 16);

	private final static Font plainFont = new Font("", Font.PLAIN, 12);

	private final static Point2f centeringAreaSize = new Point2f(
			Param.PENALTY_AREA_LENGTH + 5f,
			((Param.PITCH_WIDTH - Param.PENALTY_AREA_WIDTH) / 2));

	private final static Point2f centeringGoalAreaSize = new Point2f(
			Param.PENALTY_AREA_LENGTH + 5f, (goalAreaSize.y));

	private final static Point2f centeringAreaCornerNW = new Point2f(
			topLeftCorner);

	private final static Point2f centeringAreaCornerSW = new Point2f(
			bottomLeftCorner.x, bottomLeftCorner.y - centeringAreaSize.y);

	private final static Point2f centeringAreaCornerNE = new Point2f(
			topRightCorner.x - centeringAreaSize.x, topRightCorner.y);

	private final static Point2f centeringAreaCornerSE = new Point2f(
			bottomRightCorner.x - centeringAreaSize.x, bottomRightCorner.y
					- centeringAreaSize.y);

	private final static Point2f centeringAreaCornerGL = new Point2f(
			leftGoalArea);

	private final static Point2f centeringAreaCornerGR = new Point2f(
			topRightCorner.x - centeringGoalAreaSize.x, rightGoalArea.y);

	private final CenteringArea centeringAreaNW = new CenteringArea(
			centeringAreaCornerNW, centeringAreaSize);

	private final CenteringArea centeringAreaSW = new CenteringArea(
			centeringAreaCornerSW, centeringAreaSize);

	private final CenteringArea centeringAreaNE = new CenteringArea(
			centeringAreaCornerNE, centeringAreaSize);

	private final CenteringArea centeringAreaSE = new CenteringArea(
			centeringAreaCornerSE, centeringAreaSize);

	private final CenteringArea centeringAreaGL = new CenteringArea(
			centeringAreaCornerGL, centeringGoalAreaSize);

	private final CenteringArea centeringAreaGR = new CenteringArea(
			centeringAreaCornerGR, centeringGoalAreaSize);

	class PlayerList {
		List playerlist = new ArrayList();

		static final int size = 3;

		public void add(Player player) {
			this.playerlist.add(player);
			if (this.playerlist.size() == size + 1) {
				this.playerlist.remove(0);
			}
		}

		public boolean isDribble() {
			// System.out.println("LIST>>>>>>>>>" + playerlist.toString());
			if (this.playerlist.size() < size) {
				return false;
			} else {
				Player[] player = new Player[size];
				for (int i = 0; i < size; i++) {
					player[i] = (Player) (this.playerlist.get(i));
				}
				boolean isSameUnum = ((player[0].side == player[1].side)
						&& (player[0].unum == player[1].unum)
						&& (player[0].side == player[2].side) && (player[0].unum == player[2].unum));
				boolean isSamePosition = ((player[0].pos
						.distance(player[1].pos) < 0.1f) && (player[1].pos
						.distance(player[2].pos) < 0.1f));

				return (isSameUnum && !isSamePosition);
			}
		}
	}

	class IsCentering {
		public IsCentering() {
		}

		public boolean isCentering(Point2f p, ClientScene clientScene) {
			ClientScene tscene = clientScene;
			Point2f p1 = new Point2f(tscene.ball.pos);
			Vector2f vel = new Vector2f(tscene.ball.vel);
			for (int i = 0; i < 15; i++) {
				vel.scale(Param.BALL_DECAY);
				p1.add(vel);
			}

			return ((CommentaryLayer.this.centeringAreaNW.contains(p)
					|| CommentaryLayer.this.centeringAreaSW.contains(p)
					|| CommentaryLayer.this.centeringAreaNE.contains(p) || CommentaryLayer.this.centeringAreaSE
					.contains(p)) && (CommentaryLayer.this.centeringAreaGR
					.contains(p1) || CommentaryLayer.this.centeringAreaGL
					.contains(p1)));
		}
	}

	class CenteringArea {
		Point2f topleft, size;

		public CenteringArea(Point2f topleft, Point2f size) {
			this.topleft = topleft;
			this.size = size;
		}

		boolean contains(Point2f p) {
			return ((p.x > this.topleft.x)
					&& (p.x < (this.topleft.x + this.size.x))
					&& (p.y > this.topleft.y) && (p.y < (this.topleft.y + this.size.y)));
		}

		void draw(Graphics g) {
			drawRect(g, this.topleft, this.size);
			/*
			 * Point2f p = fieldToScreen(topleft); Point2f s = new
			 * Point2f(size); s.scale(magnify); g.drawRect((int)p.x, (int)p.y,
			 * (int)s.x, (int)s.y);
			 */
		}
	}

	private float floatWhichMin(float x, float y) {
		if (x <= y) {
			return x;
		} else {
			return y;
		}
	}
}
