/*LICENSE_BEGIN
 * ******************************************************************
 * All original parts of SoccerScope2003 have copyright (2003) by
 * Team YowAI, represented by Mr. Shuhei Shiota
 * The Department of Creative Informatics
 * The Graduate School of Information Science and Technology
 * The University of Tokyo
 * 
 * Additional classes (except libraries in /lib) were developed by 
 * the Dainamite Robocup-Team of the DAI-Labor, TU-Berlin 
 * (www.dai-labor.de)
 * 
 * Extended/new classes/packages are:
 *   soccerscope.analyzer.LogfileAnalyzer
 *   soccerscope.daistatistic.*
 *   soccerscope.daistatistic.charts.*
 *   soccerscope.daistatistic.tools.*
 *   soccerscope.db.*
 *   soccerscope.dialog.*
 *   soccerscope.model.PlayerWorldModel
 *   soccerscope.model.SimplePlayer
 *   soccerscope.net.AgentConnector
 *   soccerscope.training.*
 *   soccerscope.training.conditions.*
 * ******************************************************************
LICENSE_END*/
package soccerscope.view.layer;

import java.awt.Color;
import java.awt.Graphics;

import soccerscope.model.ClientScene;
import soccerscope.model.Param;
import soccerscope.util.geom.Point2f;
import soccerscope.view.FieldPane;

public class OffsideLayer extends Layer {

	public OffsideLayer(FieldPane fieldPane, boolean enable) {
		super(fieldPane, enable);
	}

	@Override
	public String getLayerName() {
		return "Offside";
	}

	private Point2f top = new Point2f(0, -Param.PITCH_HALF_WIDTH);

	private Point2f bottom = new Point2f(0, Param.PITCH_HALF_WIDTH);

	@Override
	public void draw(Graphics g) {
		if (!this.enable)
			return;

		ClientScene clientScene = this.fieldPane.getScene();
		Color leftColor, rightColor;
		if (clientScene.right.offside)
			leftColor = Color.red;
		else
			leftColor = Color.green;

		if (clientScene.left.offside)
			rightColor = Color.red;
		else
			rightColor = Color.green;

		g.setColor(leftColor);
		this.top.x = clientScene.left.offsideline;
		this.bottom.x = clientScene.left.offsideline;
		drawLine(g, this.top, this.bottom);

		g.setColor(rightColor);
		this.top.x = clientScene.right.offsideline;
		this.bottom.x = clientScene.right.offsideline;
		drawLine(g, this.top, this.bottom);

		float playerMagnify = this.fieldPane.getPlayerMagnify();
		for (int i = 0; i < Param.MAX_PLAYER * 2; i++) {
			if (!clientScene.player[i].isEnable())
				continue;

			float playerR = clientScene.player[i].getSize() * playerMagnify;
			// draw mode
			if (clientScene.player[i].offside) {
				g.setColor(Color.white);
				drawCircle(g, clientScene.player[i].pos, playerR + 0.1f);
			}
		}
	}
}
